#!/usr/bin/env python

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import time
import os
from raspisump import webchart

user = os.getlogin()


def main():
    """Pass variables to webchart.py"""
    year = time.strftime("%Y")
    month = time.strftime("%m")
    today = time.strftime("%Y%m%d")
    homedir = "/home/" + user + "/raspi-sump/"
    webchart.create_folders(year, month, homedir)
    webchart.create_chart(homedir)
    webchart.copy_chart(year, month, today, homedir)


if __name__ == "__main__":
    main()



import os
from datetime import datetime
from unittest import TestCase

from raspisump import log


class TestLoggingFunctions(TestCase):
    def setUp(self):
        self.test_user = os.getlogin()

    def test_log_event(self):
        """Test that notifications work correctly"""
        logfile = "test_log.csv"
        notification = "Test notification"

        log.log_event(logfile, notification)

        expected_logfile = f"/home/{self.test_user}/raspi-sump/logs/{logfile}"
        with open(expected_logfile) as f:
            log_content = f.read()

        expected_line = (
            datetime.now().strftime("%Y-%m-%d %H:%M:%S,") + notification + "\n"
        )
        self.assertIn(expected_line, log_content)
        os.remove(expected_logfile)

    def test_log_reading(self):
        """Test that waterlevel readings work correctly"""
        water_depth = 10.5
        logreading = "test-waterlevel"
        log.log_reading(logreading, water_depth)

        expected_filename = f"/home/{self.test_user}/raspi-sump/csv/{logreading}-{datetime.now().strftime('%Y%m%d')}.csv"
        with open(expected_filename) as f:
            csv_content = f.read()

        expected_line = datetime.now().strftime("%H:%M:%S,") + str(water_depth) + "\n"
        self.assertIn(expected_line, csv_content)
        os.remove(expected_filename)



#!/usr/bin/env python

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

from raspisump import checkpid, log


def main():
    """run checkpid.py module to restart Raspi-Sump if the rsump.py process is
    stopped or has spawned multiple processes."""
    process = "/usr/local/bin/rsump.py"
    checkpid.check_pid(process)


if __name__ == "__main__":
    print("rsumpmonitor.py is depracated.  Switch to systemd.")
    print(
        "https://github.com/alaudet/raspi-sump/blob/master/docs/upgrade_to_version_1.9.md"
    )
    log.log_event(
        "info_log",
        "Deprecation Warning - rsumpmonitor.py is depracated. Switch to systemd",
    )
    main()



"""Send an email on command to test."""

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import smtplib
from raspisump import alerts, config_values


configs = config_values.configuration()


def test_email_content():
    """Build the contents of test email body."""

    time_of_day = alerts.current_time()
    hostname = alerts.host_name()

    subject = "Subject: Raspi-Sump Email Test"
    message = "Raspi-Sump Test Email"

    return "\r\n".join(
        (
            f"From: {configs['email_from']}",
            f"To: {configs['email_to']}",
            f"{subject}",
            "",
            f"{hostname} - {time_of_day} - {message}.",
        )
    )


def test_email():
    """Send test email only."""
    recipients = configs["email_to"].split(", ")
    email_body = test_email_content()
    if configs["smtp_ssl"] == 1:
        server = smtplib.SMTP_SSL(configs["smtp_server"])
    elif configs["smtp_tls"] == 1:
        server = smtplib.SMTP(configs["smtp_server"])
        server.starttls()
    else:
        server = smtplib.SMTP(configs["smtp_server"])

    if configs["smtp_authentication"] == 1:
        server.login(configs["username"], configs["password"])

    server.sendmail(configs["email_from"], recipients, email_body)
    server.quit()



"""Monitor the health of a process and restart it if it has stopped
or if it has spawned multiple processes.

This module is now deprecated in favour of systemd for restarting services."""

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import subprocess
from raspisump import log


def check_pid(process):
    """Check status of process."""
    cmdp1 = "ps aux"
    cmdp2 = "grep -v grep"
    cmdp3 = "grep -v sudo"
    cmdp4 = "grep -c " + process
    cmdp1list = cmdp1.split(" ")
    cmdp2list = cmdp2.split(" ")
    cmdp3list = cmdp3.split(" ")
    cmdp4list = cmdp4.split(" ")
    part1 = subprocess.Popen(cmdp1list, stdout=subprocess.PIPE)
    part2 = subprocess.Popen(cmdp2list, stdin=part1.stdout, stdout=subprocess.PIPE)
    part1.stdout.close()
    part3 = subprocess.Popen(cmdp3list, stdin=part2.stdout, stdout=subprocess.PIPE)
    part2.stdout.close()
    part4 = subprocess.Popen(cmdp4list, stdin=part3.stdout, stdout=subprocess.PIPE)
    part3.stdout.close()
    number_of_processes = int(part4.communicate()[0])
    if number_of_processes == 0:
        log.log_event("error_log", "ERROR - rsump.py process stopped, restarting")
        restart(process)
    elif number_of_processes == 1:
        exit(0)
    else:
        log.log_event(
            "error_log",
            "ERROR - Multiple rsump.py processes...killing all and restarting",
        )
        kill_start(process)


def restart(process):
    """Restart process"""
    restart_cmd = process + " &"
    restart_now = restart_cmd.split(" ")
    subprocess.Popen(restart_now)
    exit(0)


def kill_start(process):
    """Kill all instances of process and initiate restart."""
    kill_cmd = "killall 09 " + process
    kill_it = kill_cmd.split(" ")
    subprocess.call(kill_it)
    restart(process)



"""Create charts for viewing on Raspberry Pi Web Server."""

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.htmlimport os

import os
import subprocess
import time
from raspisump import todaychart


def create_folders(year, month, homedir):
    """Check if folders exist in charts folder and create them if they don't"""
    if not os.path.isdir(f"{homedir}charts/{year}/"):
        _year = f"mkdir {homedir}charts/{year}"
        create_year = _year.split(" ")
        subprocess.call(create_year)

    if not os.path.isdir(f"{homedir}charts/{year}/{month}/"):
        _month = f"mkdir {homedir}charts/{year}/{month}"
        create_month = _month.split(" ")
        subprocess.call(create_month)


def create_chart(homedir):
    """Create a chart of sump pit activity and save to web folder"""
    csv_file = f"{homedir}csv/waterlevel-{time.strftime('%Y%m%d')}.csv"
    filename = f"{homedir}charts/today.png"
    todaychart.graph(csv_file, filename)


def copy_chart(year, month, today, homedir):
    """Copy today.png to year/month/day folder for web viewing"""
    copy_cmd = (
        f"cp {homedir}charts/today.png {homedir}charts/{year}/{month}/{today}.png"
    )
    copy_file = copy_cmd.split(" ")
    subprocess.call(copy_file)



import os
from unittest import TestCase
import raspisump.reading as reading
import raspisump.alerts as alerts
import raspisump.heartbeat as heartbeat
import raspisump.config_values as config_values

user = os.getlogin()

configs = config_values.configuration()


class TestRaspisump(TestCase):
    def test_water_reading(self):
        """Test that a proper reading is being returned."""
        pit_depth = configs["pit_depth"]
        value = reading.water_reading()
        self.assertIsInstance(value, float)
        self.assertLessEqual(value, pit_depth)

    def test_unit_types(self):
        """Test that a proper unit type is being selected."""
        measurement = alerts.unit_types()
        self.assertIsInstance(measurement, str)
        self.assertIn(measurement, ["inches", "centimeters"])

    def test_email_content(self):
        """Test that the right email alert is being returned."""
        water_depth = 35
        email_contents = alerts.email_content(water_depth)
        self.assertIsInstance(email_contents, str)
        self.assertRegex(email_contents, r"^From:")
        self.assertRegex(email_contents, r"Low Water|Sump Pump")

    def test_heartbeat_content(self):
        """Test that the right email heartbeat is being returned."""
        heartbeat_email_contents = heartbeat.heartbeat_email_content()
        self.assertIsInstance(heartbeat_email_contents, str)
        self.assertRegex(heartbeat_email_contents, r"^From:")
        self.assertRegex(heartbeat_email_contents, r"Raspi")

    def test_heartbeat_last_row(self):
        """Test last heartbeat time is correct"""
        heartbeat_log = f"/home/{user}/raspi-sump/logs/heartbeat_log"
        if os.path.isfile(heartbeat_log):
            last_alert = heartbeat.get_last_alert_time()
            self.assertIsInstance(last_alert, str)
            self.assertEqual(len(last_alert), 19)

    def test_hostname_return(self):
        """Test that hostname is being returned for alert."""
        hostname = alerts.host_name()
        self.assertIsInstance(hostname, str)

    def test_config_dict(self):
        """Test that the config is returned as a dict with the expected length"""
        self.assertIsInstance(configs, dict)
        self.assertEqual(len(configs), 20)

    def test_key_existence(self):
        """Test that all keys are present in the dict"""
        keys_to_check = [
            "critical_water_level",
            "pit_depth",
            "reading_interval",
            "temperature",
            "trig_pin",
            "echo_pin",
            "unit",
            "email_to",
            "email_from",
            "smtp_authentication",
            "smtp_tls",
            "smtp_server",
            "username",
            "password",
            "alert_when",
            "alert_interval",
            "smtp_ssl",
            "heartbeat",
            "heartbeat_interval",
            "line_color",
        ]

        for key in keys_to_check:
            self.assertIn(key, configs.keys())

    def test_dict_value_types(self):
        """Test proper value types in returned dict"""
        expected_values = {
            "critical_water_level": int,
            "pit_depth": int,
            "reading_interval": int,
            "temperature": int,
            "trig_pin": int,
            "echo_pin": int,
            "unit": str,
            "email_to": str,
            "email_from": str,
            "smtp_authentication": int,
            "smtp_tls": int,
            "smtp_server": str,
            "username": str,
            "password": str,
            "alert_when": str,
            "alert_interval": int,
            "smtp_ssl": int,
            "heartbeat": int,
            "heartbeat_interval": int,
            "line_color": str,
        }

        for key, expected_type in expected_values.items():
            self.assertIsInstance(configs[key], expected_type)



#!/usr/bin/env python

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import time
import os
from raspisump import reading, log, config_values

user = os.getlogin()
configs = config_values.configuration()
reading_interval = configs["reading_interval"]


def handle_gpio_error():
    error_message = "ERROR -- Cannot access gpio pins. Make sure the user is part of the gpio group."
    print(error_message)
    log.log_event("error_log", error_message)


def main():
    if reading_interval == 0:
        try:
            reading.water_depth()
        except RuntimeError:
            handle_gpio_error()
    else:
        while True:
            try:
                reading.water_depth()
                time.sleep(reading_interval)
            except RuntimeError:
                handle_gpio_error()
                exit(0)


if __name__ == "__main__":
    main()



"""Send Heartbeat alert to test that email notifications are working."""

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import os
import time
import smtplib
from datetime import datetime, timedelta
from collections import deque
import csv
from raspisump import log, alerts, config_values


user = os.getlogin()

configs = config_values.configuration()


def get_last_alert_time():
    """Retrieve the last alert time string from logfile"""
    heartbeat_log = "/home/" + user + "/raspi-sump/logs/heartbeat_log"
    with open(heartbeat_log, "rt") as f:
        last_row = deque(csv.reader(f), 1)[0]
        return last_row[0]


def heartbeat_email_content():
    """Build the contents of email body which will be sent as an alert"""
    heartbeat_interval_time = configs["heartbeat_interval"]
    current_time = time.strftime("%Y-%m-%d %H:%M:%S")
    last_alert = datetime.strptime(current_time, "%Y-%m-%d %H:%M:%S")
    future_date = last_alert + timedelta(minutes=heartbeat_interval_time + 1)

    weekday = future_date.strftime("%A")[0:3]
    month = future_date.strftime("%B")
    hour = future_date.strftime("%I")
    minute = future_date.strftime("%M")
    am_pm = future_date.strftime("%p").lower()

    time_of_day = alerts.current_time()
    hostname = alerts.host_name()
    subject = "Subject: Raspi-Sump Heartbeat Notification"
    message = "Raspi-Sump Email Notifications Working"

    return "\r\n".join(
        (
            f"From: {configs['email_from']}",
            f"To: {configs['email_to']}",
            f"{subject}",
            "",
            f"{hostname} - {time_of_day} - {message}.",
            f"Next heartbeat: {weekday} {month} {future_date.day} at {hour}:{minute} {am_pm}",
        )
    )


def heartbeat_alerts():
    """Send heartbeat email alert if water level greater
    than critical distance."""
    recipients = configs["email_to"].split(", ")
    email_body = heartbeat_email_content()

    if configs["smtp_ssl"] == 1:
        server = smtplib.SMTP_SSL(configs["smtp_server"])
    elif configs["smtp_tls"] == 1:
        server = smtplib.SMTP(configs["smtp_server"])
        server.starttls()
    else:
        server = smtplib.SMTP(configs["smtp_server"])

    if configs["smtp_authentication"] == 1:
        server.login(configs["username"], configs["password"])

    server.sendmail(configs["email_from"], recipients, email_body)
    server.quit()


def determine_if_heartbeat():
    """Determine if a heartbeat notification is required and if so, send
    the notification."""

    heartbeat_log = "/home/" + user + "/raspi-sump/logs/heartbeat_log"
    if not os.path.isfile(heartbeat_log):
        heartbeat_alerts()
        log.log_event("heartbeat_log", "Heartbeat Email Sent")

    else:
        heartbeat_interval_time = configs["heartbeat_interval"]
        last_email_sent = get_last_alert_time()
        current_time = time.strftime("%Y-%m-%d %H:%M:%S")
        last_heartbeat_time = datetime.strptime(last_email_sent, "%Y-%m-%d %H:%M:%S")
        time_now = datetime.strptime(current_time, "%Y-%m-%d %H:%M:%S")
        delta = time_now - last_heartbeat_time
        minutes_passed = int((delta).total_seconds() / 60)

        if minutes_passed >= heartbeat_interval_time:
            heartbeat_alerts()
            log.log_event("heartbeat_log", "Heartbeat Email Sent")
        else:
            pass



"""Graph sump pit activity."""

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import time
import numpy as np
import matplotlib as mpl

mpl.use("Agg")
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from matplotlib import rcParams
from raspisump import config_values

rcParams.update({"figure.autolayout": True})


configs = config_values.configuration()


rcParams["date.autoformatter.minute"] = "%H:%M:%S"
rcParams["date.autoformatter.hour"] = "%H:%M:%S"


def graph(csv_file, filename):
    """Create a line graph from a two column csv file."""

    unit = configs["unit"]

    date, value = np.loadtxt(
        csv_file,
        delimiter=",",
        unpack=True,
        converters={0: lambda x: mdates.datestr2num(x.decode("utf8"))},
    )

    fig = plt.figure(figsize=(10, 3.5))

    fig.add_subplot(111, facecolor="white", frameon=False)

    rcParams.update({"font.size": 9})
    plt.plot_date(
        x=date,
        y=value,
        ls="solid",
        linewidth=2,
        color="#" + configs["line_color"],
        fmt=":",
    )
    title = f"Water Level {time.strftime('%Y-%m-%d %H:%M')}"
    title_set = plt.title(title)
    title_set.set_fontsize(20.0)
    title_set.set_y(1.09)
    plt.subplots_adjust(top=0.86)

    if unit == "imperial":
        plt.ylabel("inches", fontsize=16)
    if unit == "metric":
        plt.ylabel("centimeters", fontsize=16)

    plt.xlabel("Time of Day", fontsize=16)
    plt.xticks(rotation=30)
    plt.grid(True, color="#ECE5DE", linestyle="solid")
    plt.tick_params(axis="x", bottom=False, top=False)
    plt.tick_params(axis="y", left=False, right=False)
    plt.savefig(filename, transparent=True, dpi=72)



""" Module to take a water_level reading."""

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

from hcsr04sensor import sensor
from raspisump import log, alerts, heartbeat, config_values

configs = config_values.configuration()


def initiate_heartbeat():
    """Initiate the heartbeat email process if needed"""
    if configs["heartbeat"] == 1:
        heartbeat.determine_if_heartbeat()
    else:
        pass


def water_reading():
    """Initiate a water level reading."""
    pit_depth = configs["pit_depth"]
    trig_pin = configs["trig_pin"]
    echo_pin = configs["echo_pin"]
    temperature = configs["temperature"]
    unit = configs["unit"]

    value = sensor.Measurement(trig_pin, echo_pin, temperature, unit)

    try:
        raw_distance = value.raw_distance(sample_wait=0.3)
    except SystemError:
        log.log_event(
            "error_log",
            "ERROR - Signal not received. Possible cable or sensor problem.",
        )
        exit(0)

    return round(value.depth(raw_distance, pit_depth), 1)


def water_depth():
    """Determine the depth of the water, log result and generate alert
    if needed.
    """

    critical_water_level = configs["critical_water_level"]
    water_depth = water_reading()
    if water_depth < 0.0:
        water_depth = 0.0
        log.log_reading("waterlevel", water_depth)
        log.log_event(
            "error_log",
            "ERROR - Negative reading adjusted to 0.0. Possible degrading sensor.",
        )
    log.log_reading("waterlevel", water_depth)

    if water_depth > critical_water_level and configs["alert_when"] == "high":
        alerts.determine_if_alert(water_depth)
    elif water_depth < critical_water_level and configs["alert_when"] == "low":
        alerts.determine_if_alert(water_depth)
    else:
        pass

    initiate_heartbeat()



"""Log waterlevel readings, restarts and alerts."""

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import time
import os

user = os.getlogin()


def log_event(logfile, notification):
    """Write event notification to a logfile"""
    _logfile = f"/home/{user}/raspi-sump/logs/{logfile}"
    with open(_logfile, "a") as f:
        f.write(f"{time.strftime('%Y-%m-%d %H:%M:%S,')}")
        f.write(f"{notification}\n")


def log_reading(logfile, water_depth):
    """Log time and water depth reading."""
    filename = f"/home/{user}/raspi-sump/csv/{logfile}-{time.strftime('%Y%m%d')}.csv"
    with open(filename, "a") as f:
        f.write(f"{time.strftime('%H:%M:%S,')}")
        f.write(f"{water_depth}\n")



"""Retrieve configuration from raspisump.conf"""

# Raspi-sump, a sump pump monitoring system.
# Al Audet
# https://www.linuxnorth.org/raspi-sump/
#
# All configuration changes should be done in raspisump.conf
# MIT License -- https://www.linuxnorth.org/raspi-sump/license.html

import os
import configparser

config = configparser.RawConfigParser()
user = os.getlogin()

config.read("/home/" + user + "/raspi-sump/raspisump.conf")


def configuration():
    """Return a dict of the configuration file"""

    configs = {
        "critical_water_level": config.getint("pit", "critical_water_level"),
        "pit_depth": config.getint("pit", "pit_depth"),
        "reading_interval": config.getint("pit", "reading_interval"),
        "temperature": config.getint("pit", "temperature"),
        "trig_pin": config.getint("gpio_pins", "trig_pin"),
        "echo_pin": config.getint("gpio_pins", "echo_pin"),
        "unit": config.get("pit", "unit"),
        "email_to": config.get("email", "email_to"),
        "email_from": config.get("email", "email_from"),
        "smtp_authentication": config.getint("email", "smtp_authentication"),
        "smtp_tls": config.getint("email", "smtp_tls"),
        "smtp_server": config.get("email", "smtp_server"),
        "username": config.get("email", "username"),
        "password": config.get("email", "password"),
    }

    # If not in raspisump.conf add to configs dict and provide a default value
    try:
        configs["alert_when"] = config.get("pit", "alert_when")
    except configparser.NoOptionError:
        configs["alert_when"] = "high"

    try:
        configs["alert_interval"] = config.getint("email", "alert_interval")
    except configparser.NoOptionError:
        configs["alert_interval"] = 5

    try:
        configs["smtp_ssl"] = config.getint("email", "smtp_ssl")
    except configparser.NoOptionError:
        configs["smtp_ssl"] = 0

    try:
        configs["heartbeat"] = config.getint("email", "heartbeat")
    except configparser.NoOptionError:
        configs["heartbeat"] = 0

    try:
        configs["heartbeat_interval"] = config.getint("email", "heartbeat_interval")
    except configparser.NoOptionError:
        configs["heartbeat_interval"] = 10080

    try:
        configs["line_color"] = config.get("charts", "line_color")
    except configparser.NoOptionError:
        configs["line_color"] = "FB921D"

    return configs
